#define MORE_EXAMPLES (0) ///< Define as 1 for more examples

/**
  \brief Generates correct typedef of a function returning int.

  The code
  \code
    typedef int Int_Function1(int, int);
  \endcode
  generates correct doxygen output:
  \code
    typedef int Int_Function1(int, int)
  \endcode
  which is OK and can be compiled.
*/
typedef int Int_Function1(int, int);

/**
  \brief Generates \b WRONG typedef of a function returning int.

  The code
  \code
    typedef int (Int_Function2)(int, int);
  \endcode
  generates \b WRONG doxygen output:
  \code
    typedef int() Int_Function2(int, int);
  \endcode
  which does not compile if used literally by a user.

  \note
    This syntax is used in FLTK: generates incorrect doxygen output with extraneous '()'.\n
    https://github.com/fltk/fltk/blob/e44a988c4a0770b7a12193cc023d1a55b5ff89e7/FL/Enumerations.H#L1174
    Real example in FLTK (https://www.fltk.org) as of Sep 8, 2023:\n
    This link may become invalid when new docs are generated.\n
    https://www.fltk.org/doc-1.4/Enumerations_8H.html#a427d30c9bd3a26e024ed6a712702e609

  The code
  \code
    typedef Fl_Color (Fl_Contrast_Function)(Fl_Color, Fl_Color, Fl_Fontsize, int);
  \endcode
  generates this wrong doxygen output:
  \code
    typedef Fl_Color() Fl_Contrast_Function(Fl_Color, Fl_Color, Fl_Fontsize, int)
  \endcode
  Note 'Fl_Color()' with '()' instead of 'Fl_Color' w/o '()'.
*/
typedef int (Int_Function2)(int, int);

#if (MORE_EXAMPLES)

/**
  This does not compile.

  The code as shown by doxygen can't be compiled, i.e. if a
  user used the code as documented it would not compile.
  Code and error message:
  \code
    typedef int(&) arr1a_t[10];
    error: expected unqualified-id before ‘)’ token
     21 | typedef int() Int_Function3(int, int);
        |             ^
  \endcode
*/
// typedef int() Int_Function2a(int, int);
// typedef int(&) arr1a_t[10];
// for doxygen if/when above lines are commented out
int dummy;

/**
  \brief Example typedefs from cppref.
  See third example of the following link (slightly modified):\n
  https://en.cppreference.com/w/cpp/language/typedef#Example

  \code
    // more complicated typedef
    typedef int int_t, *intp_t, (&fp)(int, long), arr1_t[10];
  \endcode
  \note
    The typedefs in the comma-separated list generate partially wrong code.
*/

// similar to the list above but not in separate lines (correct)
typedef int int2_t;             ///< see int_t
typedef int *int2p_t;           ///< see intp_t
typedef int (&f2p)(int, long);  ///< see fp(int, long)
typedef int arr2_t[10];         ///< see arr1_t

/**
  \brief My int function.
*/
int my_int_function(int a, int b) {
  return a + b;
}

/**
  \brief My second int function.
*/
int my_int_function2(int a, int b) {
  return a + b;
}

static Int_Function1 *f1 = my_int_function;  ///< example, not relevant for doxygen
static Int_Function2 *f2 = my_int_function2; ///< example, not relevant for doxygen

int main(int argc, char **argv) {
  return my_int_function(3, my_int_function2(2, 3));
}

#endif
